@@ -92,8 +92,13 @@ static const struct base_type {
 	{ "FORMAT8",		ft_FORMAT8,		SETS_FORMAT,	1},
 	{ "BE32",		ft_BE32,		ALLOWS_CONSTANTS,	4},
 	{ "FRACTION16_16",	ft_FRACTION16_16,	0,	4},
+	{ "FRACTION32_32",	ft_FRACTION32_32,	0,	8},
 	{ "FIXED",		ft_FIXED,		0,	4},
+	{ "FIXED1616",		ft_FIXED,		0,	4},
+	{ "FIXED3232",		ft_FIXED3232,		0,	8},
 	{ "LISTofFIXED",	ft_LISTofFIXED,		USES_STORE|SETS_NEXT,	0},
+	{ "LISTofFIXED1616",	ft_LISTofFIXED,		USES_STORE|SETS_NEXT,	0},
+	{ "LISTofFIXED3232",	ft_LISTofFIXED3232,	USES_STORE|SETS_NEXT,	0},
 	{ "FLOAT32",		ft_FLOAT32,		0,	4},
 	{ "LISTofFLOAT32",    	ft_LISTofFLOAT32,	USES_STORE|SETS_NEXT,	0},
 	{ "PUSH8",		ft_PUSH8,		PUSHES,	1},
@@ -177,13 +182,14 @@ struct namespace {
 		bool special;
 		struct variable *request, *response;
 	} *requests;
-	int num_events;
+	int num_events[event_COUNT];
 	struct event_data {
 		const char *name;
 		int number;
 		bool unsupported;
+		bool special;
 		struct variable *event;
-	} *events;
+	} *events[event_COUNT];
 	int num_errors;
 	const char **errors;
 	struct variable *setup;
@@ -1377,17 +1383,17 @@ static void parse_response(struct parser *parser, bool template) {
 		parse_parameters(parser, v, false);
 }
 
-static struct event_data *find_event(struct parser *parser, const char *name) {
+static struct event_data *find_event(struct parser *parser, enum event_type et, const char *name) {
 	struct namespace *n = parser->current->namespace;
-	int i;
+	struct event_data *e;
+	int i, num = n->num_events[et];
 
 	if( parser->error )
 		return NULL;
-	if( n->requests == NULL || n->num_requests <= 0 ) {
+	if( n->events[et] == NULL || num <= 0 ) {
 		return NULL;
 	}
-	for( i = 0 ; i < n->num_events ; i++ ) {
-		struct event_data *e = &n->events[i];
+	for( i = 0, e = n->events[et] ; i < num ; i++, e++ ) {
 		if( e->name == NULL )
 			continue;
 		if( strcmp(name, e->name) == 0 )
@@ -1396,17 +1402,19 @@ static struct event_data *find_event(struct parser *parser, const char *name) {
 	return NULL;
 }
 
-static void parse_events(struct parser *parser) {
+static void parse_events(struct parser *parser, enum event_type et) {
 	struct namespace *n = parser->current->namespace;
 	long firstline = parser->current->lineno;
 
 	assert( n != NULL );
 	if( n->extension == NULL && strcmp(n->name, "core") != 0 ) {
-		error(parser, "'EVENTS' only allowed in extension or namespace 'core'!");
+		error(parser, "'%sEVENTS' only allowed in extension or namespace 'core'!",
+				(et==event_xge)?"XG":"");
 		return;
 	}
-	if( n->events != NULL || n->num_events != 0 ) {
-		error(parser, "multiple 'EVENTS' in the same namespace!");
+	if( n->events[et] != NULL || n->num_events[et] != 0 ) {
+		error(parser, "multiple '%sEVENTS' in the same namespace!",
+				(et==event_xge)?"XG":"");
 		return;
 	}
 
@@ -1417,7 +1425,7 @@ static void parse_events(struct parser *parser) {
 
 		memset(&event, 0, sizeof(event));
 		/* TODO: allow excplicit setting */
-		event.number = n->num_events;
+		event.number = n->num_events[et];
 
 		name = get_const_token(parser, false);
 		if( name == NULL )
@@ -1441,7 +1449,11 @@ static void parse_events(struct parser *parser) {
 			}
 		}
 		while( (attribute = get_const_token(parser, true)) != NULL ) {
-			if( strcmp(attribute, "UNSUPPORTED") == 0 )
+			if( strcmp(attribute, "SPECIAL") == 0 ) {
+				if( event.name == NULL )
+					error(parser, "SPECIAL not possible with UNKNOWN!");
+				event.special = true;
+			} else if( strcmp(attribute, "UNSUPPORTED") == 0 )
 				event.unsupported = true;
 			else if( attribute[0] == '/' && attribute[1]  == '*' ) {
 				char *e;
@@ -1469,17 +1481,19 @@ static void parse_events(struct parser *parser) {
 						attribute);
 			}
 		}
-		newevents = realloc(n->events, sizeof(struct event_data)*(n->num_events+1));
+		newevents = realloc(n->events[et],
+				sizeof(struct event_data)*(n->num_events[et]+1));
 		if( newevents == NULL ) {
 			oom(parser);
 			break;
 		}
-		n->events = newevents;
-		newevents[n->num_events] = event;
-		n->num_events++;
+		n->events[et] = newevents;
+		newevents[n->num_events[et]] = event;
+		n->num_events[et]++;
 	}
 }
-static void parse_event(struct parser *parser, bool template) {
+
+static void parse_event(struct parser *parser, bool template, enum event_type et) {
 	const char *name, *attribute;
 	struct variable *v = NULL;
 	bool complete = false;
@@ -1507,16 +1521,18 @@ static void parse_event(struct parser *parser, bool template) {
 		if( v == NULL )
 			return;
 	}
-	event = find_event(parser, name);
+	event = find_event(parser, et, name);
 	if( template ) {
 		if( event != NULL ) {
-			error(parser, "'%s' cannot be the name for a templateEvent as it is already the name for an event in EVENTS!\n", name);
+			error(parser, "'%s' cannot be the name for a templateEvent as it is already the name for an event in EVENTS or XGEVENTS!\n", name);
 			return;
 		}
 	} else {
 		if( event == NULL ) {
-			error(parser, "EVENT '%s' not listed in previous EVENTS!\n",
-					name);
+			error(parser, "%sEVENT '%s' not listed in previous %sEVENTS!\n",
+					(et == event_xge)?"XG":"",
+					name,
+					(et == event_xge)?"XG":"");
 			return;
 		}
 		event->event = v;
@@ -1591,6 +1607,8 @@ static void parse_struct(struct parser *parser, bool list) {
 	name = get_const_token(parser, false);
 	v = add_var(parser, "", name, &name, vt_struct);
 	vl = add_var(parser, "LISTof", name, NULL, vt_type);
+	if( !v || !vl )
+		return;
 	vl->t.data = v;
 	v->refcount++;
 	if( !list ) {
@@ -1966,11 +1984,11 @@ static void check_namespace_complete(struct parser *parser) {
 				error(parser, "Expected 'RESPONSE %s'!", r->name);
 		}
 	}
-	if( n->events != NULL ) {
+	if( n->events[event_normal] != NULL ) {
 		int i;
 
-		for( i = 0 ; i < n->num_events ; i++ ) {
-			const struct event_data *e = &n->events[i];
+		for( i = 0 ; i < n->num_events[event_normal] ; i++ ) {
+			const struct event_data *e = &n->events[event_normal][i];
 
 			if( e->name == NULL )
 				continue;
@@ -1978,6 +1996,18 @@ static void check_namespace_complete(struct parser *parser) {
 				error(parser, "Missing 'EVENT %s'!", e->name);
 		}
 	}
+	if( n->events[event_xge] != NULL ) {
+		int i;
+
+		for( i = 0 ; i < n->num_events[event_xge] ; i++ ) {
+			const struct event_data *e = &n->events[event_xge][i];
+
+			if( e->name == NULL )
+				continue;
+			if( !e->unsupported && e->event == NULL )
+				error(parser, "Missing 'XGEVENT %s'!", e->name);
+		}
+	}
 }
 
 struct parser *parser_init(void) {
@@ -2035,7 +2065,9 @@ bool translate(struct parser *parser, const char *name) {
 		} else if( command_is("REQUESTS") ) {
 			parse_requests(parser);
 		} else if( command_is("EVENTS") ) {
-			parse_events(parser);
+			parse_events(parser, event_normal);
+		} else if( command_is("XGEVENTS") ) {
+			parse_events(parser, event_xge);
 		} else if( command_is("ERRORS") ) {
 			parse_errors(parser);
 		} else if( command_is("TYPE") ) {
@@ -2059,9 +2091,11 @@ bool translate(struct parser *parser, const char *name) {
 		} else if( command_is("REQUEST") ) {
 			parse_request(parser, false);
 		} else if( command_is("templateEVENT") ) {
-			parse_event(parser, true);
+			parse_event(parser, true, event_normal);
 		} else if( command_is("EVENT") ) {
-			parse_event(parser, false);
+			parse_event(parser, false, event_normal);
+		} else if( command_is("XGEVENT") ) {
+			parse_event(parser, false, event_xge);
 		} else {
 			error(parser, "Unknown command '%s'\n", command);
 		}
@@ -2097,12 +2131,16 @@ bool parser_free(struct parser *parser) {
 			variable_unref(ns->requests[i].response);
 		}
 		free(ns->requests);
-		for( i = 0 ; i < ns->num_events ; i++ ) {
-			variable_unref(ns->events[i].event);
+		for( i = 0 ; i < ns->num_events[event_normal] ; i++ ) {
+			variable_unref(ns->events[event_normal][i].event);
+		}
+		for( i = 0 ; i < ns->num_events[event_xge] ; i++ ) {
+			variable_unref(ns->events[event_xge][i].event);
 		}
 		if( ns->setup != NULL )
 			variable_unref(ns->setup);
-		free(ns->events);
+		free(ns->events[event_normal]);
+		free(ns->events[event_xge]);
 		free(ns->errors);
 		free(ns->used);
 		free(ns);
@@ -2294,9 +2332,11 @@ static const struct request *finalize_requests(struct parser *parser, struct nam
 		if( !ns->requests[i].special )
 			continue;
 		assert( rs[i].name != NULL );
-		if( strcmp(ns->name, "core") != 0 )
-			continue;
-		if( strcmp(rs[i].name, "QueryExtension") == 0 ) {
+		if( strcmp(ns->name, "core") != 0 ) {
+			fprintf(stderr, "No specials available in namespace '%s'!\n",
+					ns->name);
+			parser->error = true;
+		} else if( strcmp(rs[i].name, "QueryExtension") == 0 ) {
 			rs[i].request_func = requestQueryExtension;
 			rs[i].reply_func = replyQueryExtension;
 		} else if( strcmp(rs[i].name, "InternAtom") == 0 ) {
@@ -2321,22 +2361,36 @@ static const struct request *finalize_requests(struct parser *parser, struct nam
 	return f;
 }
 
-static const struct event *finalize_events(struct parser *parser, struct namespace *ns) {
+static const struct event *finalize_events(struct parser *parser, struct namespace *ns, enum event_type et) {
 	struct event *es;
 	const struct event *f;
 	int i;
 
-	es = calloc(ns->num_events, sizeof(struct event));
+	es = calloc(ns->num_events[et], sizeof(struct event));
 	if( es == NULL ) {
 		oom(parser);
 		return NULL;
 	}
-	for( i = 0 ; i < ns->num_events ; i++ ) {
-		es[i].name = ns->events[i].name;
+	for( i = 0 ; i < ns->num_events[et] ; i++ ) {
+		es[i].name = ns->events[et][i].name;
 		es[i].parameters = variable_finalize(parser,
-				ns->events[i].event);
+				ns->events[et][i].event);
+		if( !ns->events[et][i].special )
+			continue;
+		assert( es[i].name != NULL );
+		if( strcmp(ns->name, "ge") != 0 && strcmp(ns->name, "core") != 0) {
+			fprintf(stderr, "No specials available in namespace '%s'!\n",
+					ns->name);
+			parser->error = true;
+		} else if( strcmp(es[i].name, "Generic") == 0 ) {
+			es[i].type = event_xge;
+		} else {
+			fprintf(stderr, "No specials available for '%s::%s'!\n",
+					ns->name, es[i].name);
+			parser->error = true;
+		}
 	}
-	f = finalize_data(es, ns->num_events * sizeof(struct event),
+	f = finalize_data(es, ns->num_events[et] * sizeof(struct event),
 		       __alignof__(struct event));
 	if( f == NULL )
 		parser->error = true;
@@ -2372,11 +2426,16 @@ void finalize_everything(struct parser *parser) {
 		parser->error = true;
 		return;
 	}
-	if( core->num_events == 0 ) {
+	if( core->num_events[event_normal] == 0 ) {
 		fputs("No core events defined!\n", stderr);
 		parser->error = true;
 		return;
 	}
+	if( core->num_events[event_xge] != 0 ) {
+		fputs("Generic Events (XGE) outside of extensions not yet supported!\n", stderr);
+		parser->error = true;
+		return;
+	}
 	if( core->num_errors == 0 ) {
 		fputs("No core errors defined!\n", stderr);
 		parser->error = true;
@@ -2415,8 +2474,10 @@ void finalize_everything(struct parser *parser) {
 			e->numsubrequests = ns->num_requests;
 			e->subrequests = finalize_requests(parser, ns,
 					unknownrequest, unknownresponse);
-			e->numevents = ns->num_events;
-			e->events = finalize_events(parser, ns);
+			e->numevents = ns->num_events[event_normal];
+			e->events = finalize_events(parser, ns, event_normal);
+			e->numxgevents = ns->num_events[event_xge];
+			e->xgevents = finalize_events(parser, ns, event_xge);
 			e->numerrors = ns->num_errors;
 			e->errors = finalize_data(ns->errors,
 					ns->num_errors*sizeof(const char*),
@@ -2437,8 +2498,8 @@ void finalize_everything(struct parser *parser) {
 	}
 	requests = finalize_requests(parser, core, unknownrequest, unknownresponse);
 	num_requests = core->num_requests;
-	events = finalize_events(parser, core);
-	num_events = core->num_events;
+	events = finalize_events(parser, core, event_normal);
+	num_events = core->num_events[event_normal];
 	errors = finalize_data(core->errors,
 			core->num_errors*sizeof(const char*),
 			__alignof__(const char*));
